/*
 *
 * Copyright (c) 2020 Texas Instruments Incorporated
 *
 * All rights reserved not granted herein.
 *
 * Limited License.
 *
 * Texas Instruments Incorporated grants a world-wide, royalty-free, non-exclusive
 * license under copyrights and patents it now or hereafter owns or controls to make,
 * have made, use, import, offer to sell and sell ("Utilize") this software subject to the
 * terms herein.  With respect to the foregoing patent license, such license is granted
 * solely to the extent that any such patent is necessary to Utilize the software alone.
 * The patent license shall not apply to any combinations which include this software,
 * other than combinations with devices manufactured by or for TI ("TI Devices").
 * No hardware patent is licensed hereunder.
 *
 * Redistributions must preserve existing copyright notices and reproduce this license
 * (including the above copyright notice and the disclaimer and (if applicable) source
 * code license limitations below) in the documentation and/or other materials provided
 * with the distribution
 *
 * Redistribution and use in binary form, without modification, are permitted provided
 * that the following conditions are met:
 *
 * *       No reverse engineering, decompilation, or disassembly of this software is
 * permitted with respect to any software provided in binary form.
 *
 * *       any redistribution and use are licensed by TI for use only with TI Devices.
 *
 * *       Nothing shall obligate TI to provide you with source code for the software
 * licensed and provided to you in object code.
 *
 * If software source code is provided to you, modification and redistribution of the
 * source code are permitted provided that the following conditions are met:
 *
 * *       any redistribution and use of the source code, including any resulting derivative
 * works, are licensed by TI for use only with TI Devices.
 *
 * *       any redistribution and use of any object code compiled from the source code
 * and any resulting derivative works, are licensed by TI for use only with TI Devices.
 *
 * Neither the name of Texas Instruments Incorporated nor the names of its suppliers
 *
 * may be used to endorse or promote products derived from this software without
 * specific prior written permission.
 *
 * DISCLAIMER.
 *
 * THIS SOFTWARE IS PROVIDED BY TI AND TI'S LICENSORS "AS IS" AND ANY EXPRESS
 * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL TI AND TI'S LICENSORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */

#include "app_capture_module.h"

char gCaptTarget[][100] =
    {
        TIVX_TARGET_CAPTURE1, TIVX_TARGET_CAPTURE2, TIVX_TARGET_CAPTURE3,
        TIVX_TARGET_CAPTURE4, TIVX_TARGET_CAPTURE5, TIVX_TARGET_CAPTURE5
    };

vx_status app_init_capture(
    vx_context context, CaptureObj *captureObj,
    tivx_raw_image_create_params_t *raw_params, uint32_t numCh,
    uint32_t vcId[], uint32_t instId, char *objName, int32_t bufq_depth)
{
    vx_status           status = VX_SUCCESS;
    tivx_raw_image      raw_image;
    vx_int32            lane, q;

    tivx_capture_params_init(&captureObj->params);

    if (captureObj->enable_error_detection)
    {
        captureObj->params.timeout        = 90;
        captureObj->params.timeoutInitial = 500;
    }

    captureObj->params.numInst  = 1;
    captureObj->params.numCh    = numCh;

    captureObj->params.instId[0u]                       = instId;
    captureObj->params.instCfg[0u].enableCsiv2p0Support = (uint32_t)vx_true_e;
    captureObj->params.instCfg[0u].numDataLanes         = 4U;

    for (lane = 0; lane < captureObj->params.instCfg[0u].numDataLanes; lane++)
    {
        captureObj->params.instCfg[0u].dataLanesMap[lane] = lane + 1;
    }

    for (q = 0; q < numCh; q++)
    {
        captureObj->params.chVcNum[q]   = vcId[q];
        captureObj->params.chInstMap[q] = instId;
    }

    captureObj->config = vxCreateUserDataObject(
        context, "tivx_capture_params_t", sizeof(tivx_capture_params_t),
        &captureObj->params);

    raw_image = tivxCreateRawImage(context, raw_params);
    for(q = 0; q < bufq_depth; q++)
    {
        captureObj->raw_image_arr[q] = vxCreateObjectArray(
            context, (vx_reference)raw_image, numCh);
    }
    tivxReleaseRawImage(&raw_image);

    /*Error detection is currently enabled only for RAW input*/
    if(captureObj->enable_error_detection)
    {
        vx_map_id map_id;
        vx_rectangle_t rect;
        vx_imagepatch_addressing_t image_addr;
        void * data_ptr;

        rect.start_x = 0;
        rect.start_y = 0;
        rect.end_x = raw_params->width;
        rect.end_y = raw_params->height;

        captureObj->error_frame_raw_image = tivxCreateRawImage(context, raw_params);

        status = tivxMapRawImagePatch((tivx_raw_image)captureObj->error_frame_raw_image, &rect, 0, &map_id, &image_addr, &data_ptr, VX_READ_ONLY, VX_MEMORY_TYPE_HOST, TIVX_RAW_IMAGE_ALLOC_BUFFER);

        if ((vx_status)VX_SUCCESS == status)
        {
            memset(data_ptr, 0x00, image_addr.stride_y*(raw_params->height+raw_params->meta_height_before+raw_params->meta_height_after));

            status = tivxUnmapRawImagePatch((tivx_raw_image)captureObj->error_frame_raw_image, map_id);
        }
    }
    else
    {
        captureObj->error_frame_raw_image = NULL;
    }

    return (status);
}

void app_deinit_capture(CaptureObj *captureObj, vx_int32 bufq_depth)
{
    vx_int32 i;

    vxReleaseUserDataObject(&captureObj->config);
    for(i = 0; i < bufq_depth; i++)
    {
        vxReleaseObjectArray(&captureObj->raw_image_arr[i]);
    }

    if(NULL != captureObj->error_frame_raw_image)
    {
        tivxReleaseRawImage(&captureObj->error_frame_raw_image);
    }
}

void app_delete_capture(CaptureObj *captureObj)
{
    if(captureObj->node != NULL)
    {
        vxReleaseNode(&captureObj->node);
    }
}

vx_status app_create_graph_capture(vx_graph graph, CaptureObj *captureObj, uint32_t target)
{
    vx_status status = VX_SUCCESS;

    captureObj->node = tivxCaptureNode(graph, captureObj->config,
        captureObj->raw_image_arr[0]);
    status = vxGetStatus((vx_reference)captureObj->node);

    if (VX_SUCCESS == status)
    {
        vxSetNodeTarget(captureObj->node, VX_TARGET_STRING, gCaptTarget[target]);
        vxSetReferenceName((vx_reference)captureObj->node, "capture_node");
    }

    return status;
}

